/*-----------------------------------------------------------------------
 TestMP2.cpp 
 
   - Windows Communications, Test TTY screens.

 This program is a 32-bit MDI(Multiple Document Interface)
 communications program, each MDI child is a terminal for a port.

 The program has several goals:
 * example to demonstrate communications coding in Win32 environment.
 * Serve as a general purpose testing vehicle for communication drivers.

LAST UPDATE: 09-04-98

9-15-97 V1.38 - add ring indicator support.

9-04-97 V1.37 - don't try rts override by button press for nt
  if automatic flow is on(nt doesn't let you do this.)

8-13-97 V1.36 - Add menu selection to override into RS485 toggle-low
  mode for RS485 product support.  Clean up WIN32 warnings.

7-08-97 V1.35 - bump up timer rate to 25ms, so program can obtain
  above 115K bps.

7-01-97 V1.34 - add holding reason button feedback(2nd set of lights
   showing holding reasons.
  
6-23-97 V1.33 - if qsizes dialog screen up, then stop reading/writting
  to port.  This provides a way to test flow control.
6-05-97 V1.32 - add version tracking, add loopback test.
5-28-97 add 921600 baud rate selection.
3-26-97 take away loopback button for non-comtrol ports.
3-5-97 add ability to do purges, set/clear break.
10-21-96 add clear-screen, cr/lf option, send-xoff, send-xon.
5-24-96  Add buttons, use timer,
1-05-96  Work on displaying modem status.
10-31-95  Made data scroll properly in window.  Fixed Dialogs.
   Added optional commrkt.h usage.
10-09-95 create dialog boxes, clean up - kpb

Copyright 1995,96,97 Comtrol(TM) Corporation.
Copyright 1998 SystemBase Co., Ltd.

|-----------------------------------------------------------------------*/
/*** #include <windowsx.h> ***/

#define STRICT
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <ctype.h>
#include <string.h>
#include "TestMP2r.h"
#include "TestMP2.h"
#include "resource.h"

// define the following macro to overcome COM9 limit with WIN3.1 driver.
// (This is not needed for the default 32-bit compilation)
// also you will have to link in the commrkt.lib import lib in the make file.
//#define NEED_OVER_COM9 1

#define PKT_WRITE_SIZE 5000
#define _export 

// #define SHOW_CLOCK 1

BOOL FAR PASCAL CloseEnumProc (HWND, LONG) ;
LRESULT FAR PASCAL _export MainWndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT FAR PASCAL _export WcomWndProc(HWND, UINT, WPARAM, LPARAM);

static void SetSettingsDlgInfo(HWND hDlg);
static void GetSettingsDlgInfo(HWND hDlg);
BOOL FAR PASCAL settings_dlg_proc(HWND hDlg, UINT message,
                         WPARAM wParam, LPARAM lParam);

BOOL FAR PASCAL OpenPortProc(HWND hDlg, UINT message,
                         WPARAM wParam, LPARAM lParam);

void init_events_buttons(HWND hDlg, DWORD ev,int rec_cnt, int send_cnt);

BOOL FAR PASCAL enter_int_dlg_proc(HWND hDlg, UINT message,
                                 WPARAM wParam, LPARAM lParam);

static int GetDlgItemNumb(HWND hDlg, int ctrl, unsigned long *num);
void poll_ports(void);
static void draw_light(HDC hdc, int on, int index, int flags);
static void draw_hold_light(HDC hdc, int index, int flags);

// modem status register defs
#define ACE_CTS  0x10  //  Clear To Send
#define ACE_DSR  0x20  // Data Set ready
#define ACE_RI   0x40  // Ring Indicator
#define ACE_RLSD 0x80  // Receive Line Signal Detect

// have to track output pins ourselves
#define MY_DTR      0x01
#define MY_RTS      0x02
#define MY_LOOPBACK 0x04
/*--------------------------  Global Variables  ---------------------*/
HINSTANCE glob_hinst = 0;      // current instance

// need to make these available in the program menu or dialog.
int   msr_cheat = 1;
//int event_cheat = 0;
int show_status = 1;

int frame_width;
int frame_height;
char gtmpstr[160];
HMENU  hMenuMain;

WCom *head_wcom = NULL; // head of list of active port windows

/*----------------------- local vars ---------------------------------*/
HMENU  hMenuMainWindow;

HWND  hwndDraw   = 0;
HWND  hwndFrame  = 0;
HWND  hwndClient = 0;

static int int_val;
static long long_val;

static HGLOBAL hmem_glob = 0;
static char far *mem_glob = 0;

WCom *glob_wcom = NULL;  // used when opening port windows
WCom *glob_dlg_wcom = NULL;

int show_crlf = 0;
char default_str[60];
int send_mess_flag = 0;
int gfSelectAllPorts = 0;

unsigned char test_str[PKT_READ_SIZE+100];
int test_str_len = 0;
DWORD PlatformId;

char *hold_buttons[] = {"CTS", "DSR", "CD",  "XOFFH", "XOFFS", "EOF", "TXIM", NULL};
int hold_buttons_x[8];
int hold_buttons_size[8];

char *buttons[] = {"CTS", "DSR", "CD",  "RI", "RTS", "DTR", "LOOP", NULL};
int buttons_x[8];
int buttons_size[8];
WORD last_port_menu;
char pickportname[42];

static char *apname = {"TestMP2"};

static char *copyright = {"Copyright (c) 1996,97 Comtrol Corp."};

/*------------------------------------------------------------------------
| WinMain -
|------------------------------------------------------------------------*/
int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
				   LPSTR lpCmdLine, int nCmdShow)
{
   HACCEL  hAccel;
   MSG      msg;
   WNDCLASS  wc;
   DWORD old_ms_time = 0;

   int i, lastx;

   glob_hinst = hInstance;

   //if (hPrevInstance)	// Other instances of app running?
   //{
   //  MessageBox(GetFocus(),"Program is already running!","",MB_OK);
   //  return 0;
   //}

           // calculate the button sizes and positions
   i = 0;
   lastx = 5;
   while (buttons[i] != NULL)
   {
     buttons_x[i] = lastx;
     buttons_size[i] = strlen(buttons[i]) * 10;
     lastx += buttons_size[i] + 2;
     if ((i==3) || (i==6))
       lastx += 5;  // offset over
     ++i;
   }

   // calculate the holding status button sizes and positions
   i = 0;
   lastx = 60;
   while (hold_buttons[i] != NULL)
   {
     hold_buttons_x[i] = lastx;
     hold_buttons_size[i] = strlen(hold_buttons[i]) * 10;
     lastx += hold_buttons_size[i] + 2;
     ++i;
   }

   if (!hPrevInstance)	// Other instances of app running?
   {
     // FRAME WINDOW
     wc.style       = CS_HREDRAW | CS_VREDRAW;	// Class style(s).
     wc.lpfnWndProc = MainWndProc;
     wc.cbClsExtra  = 0;	// No per-class extra data.
     wc.cbWndExtra  = 0;	// No per-window extra data.
     wc.hInstance   = hInstance;	// Application that owns the class.
     wc.hIcon       = LoadIcon(hInstance, MAKEINTRESOURCE(TestMP2));
     wc.hCursor     = LoadCursor(NULL, IDC_ARROW);
     wc.hbrBackground = (HBRUSH) (COLOR_APPWORKSPACE + 1);
     wc.lpszMenuName  = NULL;  // Name of menu resource in .RC file.
     wc.lpszClassName = "WcomFrame";	// Name used in call to CreateWindow.
     RegisterClass(&wc);

     // Draw WINDOW
     wc.style       = CS_HREDRAW | CS_VREDRAW | CS_OWNDC; // Class style(s).
     wc.lpfnWndProc = WcomWndProc;
     wc.cbClsExtra  = 0;	// No per-class extra data.
     wc.cbWndExtra  = sizeof(void *);
     wc.hInstance   = hInstance;	// Application that owns the class.
     wc.hIcon       = LoadIcon(hInstance, MAKEINTRESOURCE(TestMP2));
     wc.hCursor     = LoadCursor(NULL, IDC_ARROW);
     wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
     wc.lpszMenuName  = NULL;	// Name of menu resource in .RC file.
     wc.lpszClassName = "WcomDraw";	// Name used in call to CreateWindow.
     RegisterClass(&wc);
   }
  /* Perform initializations that apply to a specific instance */

   // following is a buildcomm() string which can be used to init the ports
   // format: "9600,n,8,1"
   default_str[0] = 0;
   if ((lpCmdLine != NULL) && (lpCmdLine[0] != 0))
   {
     i=0;
     while (lpCmdLine[i] != 0)
     {
       default_str[i] = lpCmdLine[i];
       ++i;
     }
     default_str[i] = 0;
     // strcpy(default_str, lpCmdLine);
   }
   else
   {
     strcpy(default_str, "9600,n,8,1");  // use this as a default
   }

           // Obtain handles to 1 possible menus & submenus
   hMenuMain = LoadMenu (glob_hinst, "WCOMAPMENU");

   hMenuMainWindow = GetSubMenu (hMenuMain, 0);

   hAccel = LoadAccelerators (glob_hinst, "WcomAccel") ;

             // Create the frame window
  OSVERSIONINFO os;  // about 150 bytes big

  os.dwOSVersionInfoSize = sizeof(os);
  if (FALSE == GetVersionEx(&os))  // nt or win95 or win32s
  {
    // mess("Platform Detect Fail: %d, [%dH]", PlatformId, PlatformId);
    PlatformId = VER_PLATFORM_WIN32_NT;
  }

	/* Create a main window for this application instance.  */
   hwndFrame = CreateWindow(
		"WcomFrame", 	               // See RegisterClass() call.
                                          // Text for window title bar.
         apname,
		 WS_OVERLAPPEDWINDOW,    // Window style.
		 CW_USEDEFAULT, CW_USEDEFAULT, // Def. horz, vert pos.
         CW_USEDEFAULT, CW_USEDEFAULT, // Default width, height
         NULL,      // No Parent Window
         hMenuMain,     // Use the window class menu.
         hInstance, // This instance owns this window.
         NULL);     // Pointer not needed.

   hwndClient = GetWindow (hwndFrame, GW_CHILD) ;

   hMenuMainWindow = GetSubMenu (hMenuMain, 0);

   ShowWindow (hwndFrame, nCmdShow) ;
   UpdateWindow (hwndFrame) ;

   while (GetMessage (&msg, NULL, 0, 0))
   {
     if (!TranslateMDISysAccel (hwndClient, &msg) &&
         !TranslateAccelerator (hwndFrame, hAccel, &msg))
     {
       TranslateMessage (&msg) ;
       DispatchMessage (&msg) ;
     }
   }
   return msg.wParam;
}

/*------------------------------------------------------------------------
| poll_ports
|------------------------------------------------------------------------*/
void poll_ports(void)
{
  WCom *wcom = head_wcom;


             // do com port polling every so often
  while (wcom != NULL)
  {
    ++wcom->ticks;
    wcom->check_com_port();

    if ((wcom->old_msr_status != wcom->msr_status) ||
        (wcom->old_mcr_status != wcom->mcr_status))
      wcom->screen_update_flag = 1;

    if (wcom->screen_update_flag)  // need to update the screen.
    {
      wcom->update_screen(0);  // update changes to screen
      wcom->screen_update_flag = 0;
    }

    wcom = wcom->next_wcom;
  }
}

/*------------------------------------------------------------------------
| MainWndProc - MDI Frame Proc.
|------------------------------------------------------------------------*/
LRESULT FAR PASCAL _export MainWndProc(HWND hwnd, UINT message,
						WPARAM wParam, LPARAM lParam)
{
// DLGPROC         DlgProcPtr; /* pointer to the setup_scr function */
 FARPROC lpProc; /* pointer to the setup_scr function */
 CLIENTCREATESTRUCT clientcreate ;
 FARPROC            lpfnEnum;
 HWND               hwndChild;
 

  switch (message)
  {
    case WM_CREATE: // Initialize Global vars
      
         // disable settings menu
      EnableMenuItem(hMenuMain,1,MF_BYPOSITION | MF_DISABLED);
         // disable windows menu
      EnableMenuItem(hMenuMain,2,MF_BYPOSITION | MF_DISABLED);

      clientcreate.hWindowMenu  = hMenuMainWindow;
      clientcreate.idFirstChild = IDM_FIRSTCHILD;
      hwndClient = CreateWindow ("MDICLIENT", NULL,
                       WS_CHILD | WS_CLIPCHILDREN | WS_VISIBLE,
                       0, 0, 0, 0, hwnd, (HMENU) 0,  glob_hinst,
                       (LPSTR) &clientcreate);

      SetTimer(hwnd, 0, 25, NULL); // setup 25ms sec timer
   
	return NULL;
  
    case WM_TIMER:
      poll_ports();
    break;

    case WM_COMMAND:	// message: command from application menu

      switch(wParam)
      {
       case IDM_CLOSE:          // Close the active window
   
         hwndChild = (HWND) (SendMessage (hwndClient,
                                          WM_MDIGETACTIVE,(WPARAM) 0, (LPARAM)0L)) ;
   
         if (SendMessage (hwndChild, WM_QUERYENDSESSION, (WPARAM) 0, (LPARAM) 0L))
                          SendMessage (hwndClient, WM_MDIDESTROY,
                                       (WPARAM)hwndChild, (LPARAM)0L) ;
       return 0;
	
	   case IDOPEN:

			lpProc = MakeProcInstance((FARPROC)OpenPortProc, glob_hinst);
//          lpProc = MakeProcInstance((FARPROC)portname_dlg_proc, glob_hinst);
			DialogBox(glob_hinst, MAKEINTRESOURCE(OPENPORT), hwnd, (DLGPROC) lpProc);
			FreeProcInstance(lpProc);

			return 0;

       case IDM_SHOWCRLF:
         show_crlf ^= 1;
         if (show_crlf)
           CheckMenuItem (hMenuMain,IDM_SHOWCRLF,  MF_CHECKED);
         else
           CheckMenuItem (hMenuMain,IDM_SHOWCRLF,  MF_UNCHECKED);
       return 0;

       case IDM_SENDMESS:
         send_mess_flag ^= 1;
         if (send_mess_flag)
           CheckMenuItem (hMenuMain,IDM_SENDMESS,  MF_CHECKED);
         else
           CheckMenuItem (hMenuMain,IDM_SENDMESS,  MF_UNCHECKED);
       return 0;

                            // Messages for arranging windows
       case IDM_TILE:
         SendMessage (hwndClient, WM_MDITILE, (WPARAM)0, (LPARAM)0L) ;
       return 0;
   
       case IDM_CASCADE:
         SendMessage (hwndClient, WM_MDICASCADE, (WPARAM)0, (LPARAM)0L) ;
       return 0;
   
       case IDM_ARRANGE:
           SendMessage (hwndClient, WM_MDIICONARRANGE, (WPARAM)0, (LPARAM)0L) ;
       return 0;
   
       case IDM_CLOSEALL:       // Attempt to close all children
   
         lpfnEnum = MakeProcInstance ((FARPROC)CloseEnumProc, glob_hinst) ;
         EnumChildWindows (hwndClient, (WNDENUMPROC) lpfnEnum, (LPARAM)0L) ;
         FreeProcInstance (lpfnEnum) ;
       return 0;
   
       case ID_MENUABOUT:
          wsprintf(gtmpstr, "TestMP version %s\n\nCopyright (c) SystemBase 1998.",
                   VERSION_STRING);
          MessageBox(hwnd, gtmpstr, "About TestMP",MB_OK | MB_ICONINFORMATION);
       return 0;

       default:            // Pass to active child
         hwndChild = (HWND) (SendMessage (hwndClient,
                               WM_MDIGETACTIVE, (WPARAM)0, (LPARAM)0L));
         if (IsWindow (hwndChild))
              SendMessage (hwndChild, WM_COMMAND, wParam, lParam);
       break;
     }
    break;

    case WM_QUERYENDSESSION:
    case WM_CLOSE:       // Attempt to close all children
      SendMessage (hwnd, WM_COMMAND, (WPARAM)IDM_CLOSEALL, (LPARAM)0L) ;
      if (NULL != GetWindow (hwndClient, GW_CHILD))
         DestroyWindow(hwnd);
    break ;   // ie, call DefFrameProc ;

    case WM_SIZE:
      frame_width = LOWORD(lParam);
      frame_height = HIWORD(lParam);
    break; // have to let default have this too!

    case WM_QUIT:
    case WM_DESTROY:  // message: window being destroyed
      PostQuitMessage(0);
    return 0 ;
  }
  return DefFrameProc (hwnd, hwndClient, message, wParam, lParam) ;
}

/*-------------------------------------------------------------------
| WCom::WCom - Constructor
|--------------------------------------------------------------------*/
WCom::WCom()
{
  hwnd = 0;
  hfont = 0;
  next_wcom = NULL;
  com_name[0] = 0;
  com_id = 0;
  ticks  = 0;
  event_ptr   = NULL;
  ev_rec_cnt  = -1;   // win16, rec-cnt notification amount, -1 = disable
  ev_send_cnt = -1;  // win16, send-cnt notification amount, -1 = disable
  event_mask  = 0;   // win16, bit flags of notifications we want
  msr_status = 0;
  mcr_status = (MY_RTS | MY_DTR);
  old_msr_status = 0xff;
  old_mcr_status = 0xff;
  screen_update_flag = 0;
  display_cur_row = 0;
  cur_row = 0;
  cur_col = 0;
  scr_size_x = 0;
  scr_size_y = 0;
  testing_flag = 0;

  in_buf[0] = 0;
  for (int i=0; i< TROWS;  i++)        /* scroll text */
  {
    text_buf[i][0] = 0;
    text_buf[i][TCOLS+2] = 0;
  }

  ms_starttime = 0;
}

/*-------------------------------------------------------------------
| WCom::~WCom - Destructor
|--------------------------------------------------------------------*/
WCom::~WCom()
{

                          // take ourselves out of open port list
  if (head_wcom == this)  // we are at start of list
    head_wcom = next_wcom;
  else  // not at start of list
  {
    WCom *wc = head_wcom;   // find ourselves in list
    while ((wc->next_wcom != this) && (wc->next_wcom != NULL))
      wc = wc->next_wcom;
    if (wc->next_wcom != NULL)
      wc->next_wcom = next_wcom;  // take ourselves out of list
  }

  if (com_id != NULL)
  {
    PurgeComm(com_id, PURGE_TXABORT);
    PurgeComm(com_id, PURGE_RXABORT);
    CloseHandle(com_id);
    CloseHandle(osRead.hEvent);
    CloseHandle(osWrite.hEvent);
    CloseHandle(osEvent.hEvent);
  }
}

/*-------------------------------------------------------------------
| WCom::OpenPort - Open the port.
|--------------------------------------------------------------------*/
int WCom::OpenPort(char *name)
{
 int stat;
  strcpy(com_name, name);

#ifdef WIN32
  char nam_str[20];
  HANDLE hport;

  strcpy(nam_str, "\\\\.\\");
  strcat(nam_str, com_name);
  hport = CreateFile(nam_str,
             GENERIC_READ | GENERIC_WRITE,
//             FILE_SHARE_READ | FILE_SHARE_WRITE,  did not work for Win95
             0,  // no sharing
             NULL,                 // no security attrs
             OPEN_EXISTING,
             FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,
             NULL);
  if (hport == INVALID_HANDLE_VALUE)
  {
    stat = GetLastError();
    im_mess1("Error from CreateFile:",stat,com_name,0);
    return 1;  // error
  }
  com_id = hport;
  osRead.hEvent = CreateEvent(NULL,    // no security
                              TRUE,    // explicit reset needed.
                              FALSE,   // initial event state
                              NULL);   // no name
  if (osRead.hEvent == NULL)
     im_mess1("Error A CreateEvent",0,NULL,0);

  osWrite.hEvent = CreateEvent( NULL,    // no security
                                TRUE,    // explicit reset needed.
                                FALSE,   // initial event state
                                NULL ) ; // no name
  if (osWrite.hEvent == NULL)
     im_mess1("Error B CreateEvent",0,NULL,0);

  memset(&osEvent,0,sizeof(OVERLAPPED));
  osEvent.hEvent = CreateEvent(NULL,    // no security
                                TRUE,    // explicit reset needed
                                FALSE,   // initial event reset
                                NULL ) ; // no name
  if (osEvent.hEvent == NULL)
    im_mess1("Error C CreateEvent",0,NULL,0);

  InPendingFlag = 0;
  OutPendingFlag = 0;
  EventPendingFlag = 0;
#else  // 16-bit
  int hport;

  hport = OpenComm((char far*)com_name, 1024, 1024);
  if (hport < 0)
  {
    im_mess1("Error from OpenComm:",(int)hport,com_name,0);
    return 1;  // error
  }
  com_id = hport;

#ifdef COMMENT_OUT
  stat = EscapeCommFunction(com_id, CLRRTS);
  if (stat != 0)
     im_mess1("Error from EscapeCommFunction:",stat,NULL,0);
  stat = EscapeCommFunction(com_id, SETRTS);
  if (stat != 0)
     im_mess1("Error from EscapeCommFunction:",stat,NULL,0);
#endif
  dcb->DCBlength = sizeof(DCB);

  //stat = EscapeCommFunction(com_id, GETMAXCOM);
  //im_mess1("GETCOMMAX:",stat,NULL,0);

  if (default_str[0] != 0)
  {
    strcpy(gtmpstr, com_name);
    strcat(gtmpstr, ":");
    strcat(gtmpstr, default_str);
    stat = BuildCommDCB(gtmpstr, &dcb);
    if (stat == 0)  // ok
    {
      stat = SetCommState(&dcb);
      if (stat != 0)
        im_mess1("Error from SetCommState A3:",stat,NULL,0);
    }
  }
  else
  {
                             // read in the current port settings
    GetCommState(com_id, &dcb);
  }


  if (msr_cheat)
  {
	  GetCommEventMask(glob_wcom->com_id, 0x2fff);
     glob_wcom->event_ptr = SetCommEventMask(glob_wcom->com_id, 0);
     if (glob_wcom->event_ptr != NULL)
     {
       glob_wcom->msr_cheat_ptr = (unsigned char far *) glob_wcom->event_ptr;
       glob_wcom->msr_cheat_ptr += 35;  // offset up to msr byte.
       glob_wcom->msr_status = *msr_cheat_ptr;
    }
    else
    {
      msr_cheat = 0;
    }
  }
#endif
  
  //PortIdentify();  // see if we can support loopback button  <------- Windows 98 COM1 Ҷ ״¿!!
  num_buttons = 6;
  // add our selves to linked list of opened ports.
  WCom *wc = head_wcom;   // save
  head_wcom = this; // set new
  next_wcom = wc;        // link to last one
  
  return 0; // ok
}

/*-------------------------------------------------------------------
| WCom::WritePort - send data out the port.
|--------------------------------------------------------------------*/
int WCom::WritePort(unsigned char *str, int cnt)
{
 int stat,j;
 DWORD dwBytesWritten;

  if (cnt == 0) cnt = strlen((char *)str);
#ifdef WIN32

  if (OutPendingFlag)         // waiting on last write, need to wait..
  {                           // wait in here:
    if ((stat = WaitForSingleObject(osWrite.hEvent, 6000)) == 0)   // ok
    {
      //  printf("Pending Ok\n");
      if ((stat = GetOverlappedResult(com_id,
                            &osWrite,
                            &dwBytesWritten, FALSE)) == FALSE)
      {
        stat = GetLastError();
        if (stat == ERROR_IO_INCOMPLETE)
             im_mess1("Timeout Comout GetOverLappedResult:",stat,NULL,0);
        else im_mess1("Error Comout GetOverLappedResult:",stat,NULL,0);
      }
      // else wait finished ok.
    }
    else
      im_mess1("ComOut Pending Failed, stat:",stat,NULL,0);
    OutPendingFlag = 0;
  }

  stat = WriteFile(com_id, str, cnt, (DWORD *)&j, &osWrite);

  if (stat == FALSE)  // error
  {
    stat = GetLastError();
    if (stat == ERROR_IO_PENDING)  // waiting for completion
      OutPendingFlag = 1;
    else
    {
      im_mess1("Com Write Error:",stat,NULL,0);
    }
  }

#else
 int i;

  i = WriteComm(com_id, str, cnt);
  if (i != cnt)
  {
    sprintf(gtmpstr,"<ERR WriteComm, sent:%d got:%d>", test_str_len, i);
    mess_str(gtmpstr);  // send to screen
  }
#endif

 return 0;
}

/*-------------------------------------------------------------------
| WCom::ReadPort - send data out the port.
|--------------------------------------------------------------------*/
int WCom::ReadPort(unsigned char *str, int want_cnt, int *got_cnt)
{
 int stat;

#ifdef WIN32
 DWORD BytesRead;

  stat = ReadFile(com_id, str, want_cnt, (DWORD *) &BytesRead, &osRead);

  if (stat == FALSE)  // error
  {
    stat = GetLastError();
    if (stat == ERROR_IO_PENDING)  // waiting for completion
    {
      if ((stat = WaitForSingleObject(osRead.hEvent, 60000)) == 0)  // ok
      {
        // printf("Comin Pending OK\n");
        if ((stat = GetOverlappedResult(com_id,
                            &osRead,
                            &BytesRead, FALSE)) == FALSE)
        {
          stat = GetLastError();
          if (stat == ERROR_IO_INCOMPLETE)
             im_mess1("Timeout Comin GetOverLappedResult:",stat,NULL,0);
          else im_mess1("Error Comin GetOverLappedResult:",stat,NULL,0);
        }
        else
        {
          // osRead.Offset += BytesRead;  // bump comfile ptr
          if (BytesRead != (DWORD)want_cnt)
          {
            im_mess1("ComRead Bytes, got:", BytesRead, ",Wanted:", want_cnt);
            *got_cnt = (int) BytesRead;
            if (BytesRead != 0)
              return 0;  // ok
          }
          else
          {
            // printf("ComIn Read OK\n");
            return 0;  // ok
          }
        }
      }
      else
      {
        *got_cnt = (int) BytesRead;
        im_mess1("ComIn Pending Failed!:", stat, ",Wanted:", want_cnt);
      }
    }
    else
        im_mess1("Error On Read!:", stat, ",Wanted:", want_cnt);
  }
  else
  {
    if (BytesRead != (DWORD)want_cnt)
      im_mess1("ComRead Error B Bytes, got:", BytesRead, ",Wanted:", want_cnt);

    *got_cnt = (int) BytesRead;
    return 0;  // ok
  }
  return 1;  // error

#else   // 16-bit Windows
  COMSTAT com_stat;
  int num_read;

    num_read = ReadComm(com_id, in_buf, want_cnt);
    if (num_read < 0)      // check for errors
    {
      num_read = -num_read;
      stat = GetCommError(com_id,&com_stat);
      sprintf(gtmpstr,"<ERR %x,ReadComm, %s>", stat, com_name);
      mess_str(gtmpstr);  // send to screen
      return 1;  //error
    }
   *got_cnt = num_read;
   return 0; // ok
#endif
}

/*------------------------------------------------------------------------
| WCom::StatusPort -
|------------------------------------------------------------------------*/
int WCom::StatusPort(PortStatus *ps)
{
#ifdef WIN32
  DWORD ret_stat,stat;
  stat = ClearCommError(com_id, &ret_stat, &cur_com_stat);
  if (stat == FALSE)
  {
    stat = GetLastError();
    sprintf(gtmpstr,"<ERR %xH,ClearCommError, %s, handle:%x>", stat, com_name, com_id);
    mess_str(gtmpstr);  // send to screen
  }
  else
  {
    ps->in_cnt = cur_com_stat.cbInQue;
    ps->out_cnt = cur_com_stat.cbOutQue;

    // check for change in comstat holding bitflags
    if ( *((DWORD *)(&old_com_stat)) != *((DWORD *)(&cur_com_stat)) )
    {
      screen_update_flag = 1;
      // old_com_stat = cur_com_stat;(let screen update do this)
    }

    if (ret_stat)
      print_comm_errors(ret_stat);
  }
  
#else
  int stat;
          // get and clear any errors, get input/output queue counts
  stat = GetCommError(com_id,&com_stat);
  if (stat)
    print_comm_errors(stat);

  if (msr_cheat)  // update our MSR(modem status register) status
    msr_status = *msr_cheat_ptr;
#endif

  return 0;
}

/*------------------------------------------------------------------------
| WCom::check_com_port -
|------------------------------------------------------------------------*/
void WCom::check_com_port(void)
{
 PortStatus ps;
 int i,j;
 int want_read;
 int got_read;
 int stat;

 static char *tst_str = {"abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"};
// static char *tst_str = {"1234567890abcdefghijklmnopqrstuvwxyz"};

  // get the com_stat record and get the # of characters available
  StatusPort(&ps);

#ifdef WIN32
  CheckEvents32();
#endif

  if (!testing_flag)
  {
  if ((send_mess_flag) && (ps.out_cnt == 0))
  {
    int send_line;

    if (test_str_len == 0)  // need to make the test string(done once)
    {
      j = 0;
      send_line = 5;
      while ((j< PKT_WRITE_SIZE) && (send_line > 0))// send out about PKT_SIZE bytes at a time.
      {
        for (i=0; i<send_line; i++)
          test_str[j++] = tst_str[i];
        if (send_line > 45)   // devise a string to send
        {
           send_line = 5;
           if (j > 700) send_line = 0;
        }
        else ++send_line;
        test_str[j++] = 0xd;
        test_str[j++] = 0xa;
        test_str[j] = 0;
      }
      test_str_len = j;
    }

    stat = WritePort(test_str, test_str_len);
  }

                           // get the number of characters available
  while (ps.in_cnt > 0)
  {
    want_read = ps.in_cnt;
    if (want_read > PKT_READ_SIZE) want_read = PKT_READ_SIZE;  // our buffer only PKT_READ_SIZE bytes big
    ps.in_cnt -= want_read;

    // read the characters
    stat = ReadPort(in_buf, want_read, &got_read);
    if (stat == 0)
    {
      //if (ps.in_cnt == 0)  // if more to read, don't waste time displaying
      {
        in_buf[got_read] = 0;
        mess_str((char *)in_buf, got_read);
      }

    }
    else
      mess_str("Error reading data");
  }
  }  // not testing
}

/*-------------------------------------------------------------------
| CloseEnumProc -
|--------------------------------------------------------------------*/
BOOL FAR PASCAL CloseEnumProc (HWND hwnd, LONG lParam)
{
  if (GetWindow (hwnd, GW_OWNER))         // check for icon title
    return 1;

  SendMessage (GetParent (hwnd), WM_MDIRESTORE, (WPARAM)hwnd, (LPARAM)0L);

  if (!SendMessage (hwnd, WM_QUERYENDSESSION, (WPARAM)0, (LPARAM)0L))
     return 1;

  SendMessage (GetParent (hwnd), WM_MDIDESTROY, (WPARAM)hwnd,(LPARAM) 0L);
    return 1;
}

/*-------------------------------------------------------------------
| WcomWndProc -
|--------------------------------------------------------------------*/
LRESULT FAR PASCAL _export WcomWndProc(HWND hwnd, UINT message,
						WPARAM wParam, LPARAM lParam)
{
 HDC hdc;
 PAINTSTRUCT ps;
 FARPROC lpProc; /* pointer to the setup_scr function */
 int i;
 char tmpstr[10];


  switch (message)
  {
    case WM_CREATE :
      SetWindowLong(hwnd, 0, (long) glob_wcom);
      glob_wcom->hwnd = hwnd;
      glob_wcom->ms_starttime = GetTickCount();

      hdc = GetDC(hwnd);
      SetBkColor(hdc, GetSysColor(COLOR_WINDOW));
      SetTextColor(hdc, GetSysColor(COLOR_WINDOWTEXT));
      ReleaseDC (hwnd, hdc);

      SetWindowText(hwnd, (char far*)glob_wcom->com_name);
      glob_wcom->mess_str("<Port Opened>");

      SetFocus(hwnd);
    return 0;

#ifndef WIN32
    case WM_COMMNOTIFY:
      glob_wcom = (WCom *) GetWindowLong(hwnd,0);

//     WPARAM wParam = specifes the device (nCid)
//     LPARAM lParam = LOWORD contains event trigger

      if (lParam & CN_EVENT)
      {
        // get and clear any events.
        glob_wcom->events = GetCommEventMask(glob_wcom->com_id, 0x2fff);

        glob_wcom->print_events(glob_wcom->events); // print events to screen

        if (!msr_cheat)  // then use data from events
        {
          if (glob_wcom->events & EV_CTS)
          {
            if (glob_wcom->events & EV_CTSS) glob_wcom->msr_status |= ACE_CTS;
            else glob_wcom->msr_status &= ~ACE_CTS;
          }
          if (glob_wcom->events & EV_DSR)
          {
            if (glob_wcom->events & EV_DSRS) glob_wcom->msr_status |= ACE_DSR;
            else glob_wcom->msr_status &= ~ACE_DSR;
          }
          if (glob_wcom->events & EV_RING)
          {
            if (glob_wcom->events & EV_RING) glob_wcom->msr_status |= ACE_RI;
            else glob_wcom->msr_status &= ~ACE_RI;
          }
          if (glob_wcom->events & EV_RLSD)
          {
            if (glob_wcom->events & EV_RLSDS) glob_wcom->msr_status |= ACE_RLSD;
            else glob_wcom->msr_status &= ~ACE_RLSD;
          }
        }
      }
      if (lParam & CN_TRANSMIT)
      {
        glob_wcom->mess_str("<CN_TRAN>");
      }
      if (lParam & CN_RECEIVE)
      {
        glob_wcom->mess_str("<CN_REC>");
      }

    return 0;
#endif

    case WM_CHAR:
      glob_wcom = (WCom *) GetWindowLong(hwnd,0);

      {
        tmpstr[0] = wParam;
        glob_wcom->WritePort((unsigned char *)tmpstr, 1);
      }
    return 0;

    case WM_SETFOCUS:
      glob_wcom = (WCom *) GetWindowLong(hwnd,0);
      ShowCursor(TRUE);
    break;

    case WM_KILLFOCUS:
      ShowCursor(FALSE);
    break;

    case WM_COMMAND:   // message: command from application menu
      glob_wcom = (WCom *) GetWindowLong(hwnd,0);
      glob_dlg_wcom = glob_wcom;

      switch(wParam)
      {
		case IDM_SETBREAK:
          EscapeCommFunction(glob_wcom->com_id, SETBREAK);
          glob_wcom->mess_str("<SETBREAK set>");
        break;

        case IDM_CLRBREAK:
          EscapeCommFunction(glob_wcom->com_id, CLRBREAK);
          glob_wcom->mess_str("<CLRBREAK set>");
        break;

        case IDM_SETXOFF:
          EscapeCommFunction(glob_wcom->com_id, SETXOFF);
          glob_wcom->mess_str("<SETXOFF set>");
        break;

        case IDM_SETXON:
          EscapeCommFunction(glob_wcom->com_id, SETXON);
          glob_wcom->mess_str("<SETXON set>");
        break;

        case IDM_CLEARSCR:
          glob_wcom->clear_scr();
        break;

        case IDM_LOOPTEST:
          glob_wcom->loop_test();
        break;

        case IDM_ALLPORTS_SETTINGS:
		case IDM_PORT_SETTINGS:
		  if (wParam == IDM_ALLPORTS_SETTINGS) 
			  gfSelectAllPorts = 1;
		  lpProc = MakeProcInstance((FARPROC)settings_dlg_proc, glob_hinst);
          DialogBox(glob_hinst, "SETTINGSDLGBOX", hwnd, (DLGPROC) lpProc);
          FreeProcInstance(lpProc);
        return 0;

        case IDM_EXIT:
          SendMessage (hwndFrame, WM_CLOSE, (WPARAM)0L, (LPARAM)0L) ;
        break;

        default:
        break;
      }
    return 0;

    case WM_PAINT:
      glob_wcom = (WCom *) GetWindowLong(hwnd,0);

      hdc = BeginPaint(hwnd, &ps);
      EndPaint (hwnd, &ps);

      glob_wcom->update_screen(1);  // update whole screen
    return 0;

    case WM_MDIACTIVATE:
      // Set the Draw menu if gaining focus
    return 0;

    case WM_QUERYENDSESSION:
    case WM_CLOSE:
//        if (IDOK != MessageBox (hwnd,"OK to close window?","Draw",
//                                MB_ICONQUESTION | MB_OKCANCEL))
//      {
//        return 0;
//      }
    break ;   // ie, call DefMDIChildProc

    case WM_MOUSEMOVE:
      glob_wcom = (WCom *) GetWindowLong(hwnd,0);
      glob_wcom->cur_x = LOWORD(lParam);
      glob_wcom->cur_y = HIWORD(lParam);
    break;

//    case WM_LBUTTONDOWN:
//    case WM_RBUTTONDOWN:
    case WM_LBUTTONUP:
    case WM_RBUTTONUP:
      glob_wcom = (WCom *) GetWindowLong(hwnd,0);
      if (glob_wcom->cur_y < 16)
      {
        i = 0;
        while ((buttons[i] != NULL) && (i < glob_wcom->num_buttons))
        {
          if ((glob_wcom->cur_x > buttons_x[i]) &&
              (glob_wcom->cur_x < buttons_x[i]+buttons_size[i]))
          {
            switch (i)
            {
              case 0:
              case 1:
              case 2:
              case 3:
        MessageBox(GetFocus(),"These are inputs, so cannot be controlled!","",MB_OK);
              break;
              case 4:
                if (PlatformId == VER_PLATFORM_WIN32_NT)
                {
                  // NT can't override rts control, in 95 you can
                  if (glob_wcom->dcb.fRtsControl >= 2) 
                     // control state, can't override
                  {
        MessageBox(GetFocus(),"Can't override RTS when in handshake mode in NT!","",MB_OK);
                     break;  // don't allow toggle
                  }
                }
          if (glob_wcom->mcr_status & MY_RTS)
               EscapeCommFunction(glob_wcom->com_id, CLRRTS);
          else EscapeCommFunction(glob_wcom->com_id, SETRTS);
          glob_wcom->mcr_status ^= MY_RTS;
              break;

              case 5:
          if (glob_wcom->mcr_status & MY_DTR)
               EscapeCommFunction(glob_wcom->com_id, CLRDTR);
          else EscapeCommFunction(glob_wcom->com_id, SETDTR);
          glob_wcom->mcr_status ^= MY_DTR;
              break;

              case 6:
               if (glob_wcom->mcr_status & MY_LOOPBACK)
               {
                 glob_wcom->LoopBack(0);  // turn off
               }
               else
               {
                 glob_wcom->LoopBack(1);  // turn on
               }
              break;
            }  // switch ()
          }  // if button hit
          ++i;
        }  // while
      }
    break;

    case WM_SIZE:
      glob_wcom = (WCom *) GetWindowLong(hwnd,0);
      glob_wcom->scr_size_x = LOWORD(lParam);
      glob_wcom->scr_size_y = HIWORD(lParam);
    break;

    case WM_DESTROY:
      glob_wcom = (WCom *) GetWindowLong(hwnd,0);
      delete glob_wcom;
    return 0;
  }
      // Pass unprocessed message to DefMDIChildProc
  return DefMDIChildProc (hwnd, message, wParam, lParam);
}

/*--------------------------------------------------------------------
| WCom::PortIdentify - We have the port open, try a few ioctl's to
    see if its a port that supports Comtrols loopback ioctl's.
    If not, then make it so loop button does not show.
|---------------------------------------------------------------------*/
int WCom::PortIdentify(void)
{
 int stat;
 DWORD nBytes = 0;

  loop_supported = 0;  // default to no loop
#ifdef WIN32
  if (PlatformId == VER_PLATFORM_WIN32_NT)
  {
          // following only in RocketPort WinNT driver currently.
   stat = DeviceIoControl(glob_wcom->com_id, IOCTL_RCKT_LOOPBACK_OFF,
                          (char *)NULL, 0, (char *)NULL, 0,  // out
                          &nBytes, 0);
   if (stat == 1)  // success
    loop_supported = 1;
  }
  else // win95 assume
  {
          // following only in RocketPort Win95 driver currently.
    stat = EscapeCommFunction(glob_wcom->com_id, 15649);  // loop off
    if (stat == 1)  // success
      loop_supported = 1;
  }
#else  // must be WIN3.1 or WFW311
  // since we use 95 driver in WFW311, we check for that in 16-bit mode
  stat = EscapeCommFunction(glob_wcom->com_id, 15649);  // loop off

  if (stat == 0)  // success
  {
    // win3.1 driver returns sucess on fail, so try to work around
    stat = EscapeCommFunction(glob_wcom->com_id, 16000);  // odd-ioctl num
    if (stat == 0)  // success
    {
      // probably have win3.1 driver installed.  Loop not supported.
    }
    else
      loop_supported = 1;
  }
#endif

  if (loop_supported)
    num_buttons = 7;
  else
    num_buttons = 6;
  return 0;
}


/*--------------------------------------------------------------------
| WCom::LoopBack -
|---------------------------------------------------------------------*/
int WCom::LoopBack(int on)
{
  DWORD nBytes = 0;
  int stat;

  if (on)
  {
    mcr_status |= MY_LOOPBACK;
#ifdef WIN32
    if (PlatformId == VER_PLATFORM_WIN32_NT)
    {
            // following only in RocketPort WinNT driver currently.
     stat = DeviceIoControl(glob_wcom->com_id, IOCTL_RCKT_LOOPBACK_ON,
                            (char *)NULL, 0, (char *)NULL, 0,  // out
                            &nBytes, 0);
    }
    else // win95 assume
            // following only in RocketPort Win95 driver currently.
      stat = EscapeCommFunction(glob_wcom->com_id, 15648);  // loop on
#else
            // following only in RocketPort Win95 driver currently.
    stat = EscapeCommFunction(glob_wcom->com_id, 15648);  // loop on
#endif
  }
  else  // turn off
  {
    mcr_status &= ~MY_LOOPBACK;
#ifdef WIN32
    if (PlatformId == VER_PLATFORM_WIN32_NT)
    {
            // following only in RocketPort WinNT driver currently.
      stat = DeviceIoControl(glob_wcom->com_id, IOCTL_RCKT_LOOPBACK_OFF,
                         (char *)NULL, 0, (char *)NULL, 0,  // out
                         &nBytes, 0);
    }
    else // win95 assume
            // following only in RocketPort Win95 driver currently.
      stat = EscapeCommFunction(glob_wcom->com_id, 15649);  // loop off
#else
            // following only in RocketPort Win95 driver currently.
    stat = EscapeCommFunction(glob_wcom->com_id, 15649);  // loop off
#endif
  }
  return 0;
}

/*--------------------------------------------------------------------
| WCom::CheckEvents32 - Check for Events and ModemStatus(Win32).
|---------------------------------------------------------------------*/
int WCom::CheckEvents32(void)
{
#ifdef WIN32
 DWORD num_something;
 DWORD stat;

 DWORD ModemStatus;
 DWORD ms_last_stat; // avoid million error windows on screen due to polling

  stat = GetCommModemStatus(com_id, &ModemStatus);
  if (stat == FALSE)
  {
    if (ms_last_stat == 0)
    {
      stat = GetLastError();
      im_mess1("GetCommModemStatus Error:",stat,0,0);
    }
  }
  ms_last_stat = stat;

  msr_status = 0;
  if (ModemStatus & MS_CTS_ON)
    msr_status |= ACE_CTS;
  if (ModemStatus & MS_DSR_ON)
    msr_status |= ACE_DSR;
  if (ModemStatus & MS_RLSD_ON)
    msr_status |= ACE_RLSD;
  if (ModemStatus & MS_RING_ON)
    msr_status |= ACE_RI;


  if (EventPendingFlag)
  {
    if ((stat = GetOverlappedResult(com_id,
                                    &osEvent,
                                    &num_something,
                                    FALSE)    // don't wait
                                          ) == FALSE)
    {
      stat = GetLastError();
      if (stat != ERROR_IO_INCOMPLETE)
      {
         im_mess1("Error Com Event GetOverLappedResult:",stat,0,0);
         EventPendingFlag = 0;
      }
    }
    else
    {
      print_events(events); // print events to screen
      EventPendingFlag = 0;
    }
  }

  if (EventPendingFlag)  // waiting
  {
    return 0;
  }

  if (event_mask == 0)
  {
    return 0;
  }

  stat = WaitCommEvent(com_id, &events, &osEvent);

  if (stat == FALSE)  // error
  {
    stat = GetLastError();
    if (stat == ERROR_IO_PENDING)  // waiting for completion
      EventPendingFlag = 1;
    else
    {
static int errs = 0;
      ++errs;
      if (errs > 4) return 1;
      im_mess1("Event Wait Error:",stat,0,0);
    }
  }
  else
  {
    print_events(events); // print events to screen
  }


#endif

  return 0;
}

/*--------------------------------------------------------------------
| WCom::print_comm_errors -
|---------------------------------------------------------------------*/
void WCom::print_comm_errors(unsigned int err)
{
  if (err & CE_BREAK)   mess_str("<ERR:BREAK>");
  if (err & CE_FRAME)   mess_str("<ERR:FRAME>");
  if (err & CE_IOE)     mess_str("<ERR:IOE>");
  if (err & CE_MODE)    mess_str("<ERR:MODE>");
  if (err & CE_OVERRUN) mess_str("<ERR:OVERRUN>");
  if (err & CE_RXOVER)  mess_str("<ERR:RXOVER>");
  if (err & CE_RXPARITY)mess_str("<ERR:RXPARITY>");
  if (err & CE_TXFULL)  mess_str("<ERR:TXFULL>");
  if (err & CE_DNS)     mess_str("<ERR:DNS>");
  if (err & CE_PTO)     mess_str("<ERR:PTO>");
  if (err & CE_OOP)     mess_str("<ERR:OOP>");
}

/*--------------------------------------------------------------------
| WCom::print_modem_state -
|---------------------------------------------------------------------*/
void WCom::print_modem_state(unsigned int modem_state)
{
#ifdef COMMENT_OUT
 char tmpstr[40];
 wsprintf(tmpstr,"<MODEM STATE: %s %s %s %s>",
 (modem_state & MS_CTS_ON)   ? " CTS"  : " cts"  ,
 (modem_state & MS_DSR_ON)   ? " DSR"  : " dsr"  ,
 (modem_state & MS_RING_ON)  ? " RING" : " ring" ,
 (modem_state & MS_RLSD_ON)  ? " RLSD" : " rlsd" );
 mess_str(tmpstr);
#endif
 mess_str("STATE!");
}

/*--------------------------------------------------------------------
| WCom::print_events -
|---------------------------------------------------------------------*/
void WCom::print_events(unsigned int events)
{
  if (events == 0)  mess_str("<EV_NONE>");
  if (events & EV_RXCHAR)  mess_str("<EV_RXCHAR>");
  if (events & EV_RXFLAG)  mess_str("<EV_RXFLAG>");
  if (events & EV_TXEMPTY) mess_str("<EV_TXEMPTY>");
  if (events & EV_CTS)     mess_str("<EV_CTS>");
  if (events & EV_DSR)     mess_str("<EV_DSR>");
  if (events & EV_RLSD)    mess_str("<EV_RLSD>");
  if (events & EV_BREAK)   mess_str("<EV_BREAK>");
  if (events & EV_ERR)     mess_str("<EV_ERR>");
  if (events & EV_RING)    mess_str("<EV_RING>");
  if (events & EV_PERR)    mess_str("<EV_PERR>");
#ifdef WIN32
  if (events & EV_RX80FULL)    mess_str("<EV_RX80FULL>");
  if (events & EV_EVENT1)    mess_str("<EV_EVENT1>");
  if (events & EV_EVENT2)   mess_str("<EV_EVENT2>");
#else
  if (events & EV_CTSS)    mess_str("<EV_CTSS>");
  if (events & EV_DSRS)    mess_str("<EV_DSRS>");
  if (events & EV_RLSDS)   mess_str("<EV_RLSDS>");
  if (events & EV_RingTe)  mess_str("<EV_RingTe>");
#endif
}

/*------------------------------------------------------------------------
| im_mess1 -
|------------------------------------------------------------------------*/
void im_mess1(char *str1, int i1, char *str2, int i2)
{
 char tmpstr[80];

 if (str2 == NULL)
		sprintf(tmpstr,"%s%d",str1,i1);
 else sprintf(tmpstr,"%s %d %s %d",str1,i1,str2,i2);

 MessageBox(hwndFrame, tmpstr, NULL, MB_OK | MB_ICONQUESTION);
}

/*------------------------------------------------------------------------
| WCom::mess_line -
|------------------------------------------------------------------------*/
void WCom::mess_line(char *str, int line_num)
{
 HDC hdc;

//  lstrcpy(text_buf, str);

  hdc = GetDC(hwnd);
  TextOut(hdc, 20, line_num*20, str, strlen(str));
  ReleaseDC(hwnd, hdc);
}

/*------------------------------------------------------------------------
| WCom::mess_num - send a number out to debug text window.
|------------------------------------------------------------------------*/
void WCom::mess_num(int num)
{
 char tmpstr[10];

  sprintf(tmpstr,":%d ",num);
  mess_str(tmpstr);
}

/*------------------------------------------------------------------------
 draw_light - draw the buttons at top of window.
|------------------------------------------------------------------------*/
static void draw_light(HDC hdc, int on, int index, int flags)
{
 HBRUSH new_brush, old_brush;
 LOGBRUSH log_brush;
 RECT rect;
 int i;
 char str[10];
 
  strcpy(str,buttons[index]);

  rect.top  = 1;
  rect.left = buttons_x[index];
  rect.bottom  = rect.top + 16;
  rect.right = rect.left + buttons_size[index];

  SelectObject(hdc,GetStockObject(BLACK_PEN));
  SelectObject(hdc,GetStockObject(LTGRAY_BRUSH));

  if (on)
  {
    log_brush.lbStyle = 0;  // solid brush
    log_brush.lbColor = RGB(128,255,128);  // green
    new_brush = CreateBrushIndirect(&log_brush);
    old_brush = (HBRUSH) SelectObject(hdc,new_brush);
    RoundRect(hdc, rect.left, rect.top, rect.right, rect.bottom,6,8);
    SelectObject(hdc,old_brush);
    DeleteObject(new_brush);
  }
  else
  {
    i=0;  // convert to lower case
    while (str[i] != 0)
    {
      //str[i++] &= ~0x30;
      str[i] = tolower(str[i]);
      ++i;
    }

    RoundRect(hdc, rect.left, rect.top, rect.right, rect.bottom,6,8);
  }

  SetBkMode(hdc,TRANSPARENT);
  SetTextColor(hdc, RGB(0,0,0)); // black
  DrawText(hdc, str, strlen(str), &rect, 1);
}

/*------------------------------------------------------------------------
 draw_hold_light - draw the lights at top of window for holding state.
   Put up a light if they are on, if off then don't display a light.
|------------------------------------------------------------------------*/
static void draw_hold_light(HDC hdc, int index, int flags)
{
 HBRUSH new_brush, old_brush;
 LOGBRUSH log_brush;
 RECT rect;
 int i;
 char str[10];
#define START_X_POS 240

  strcpy(str,hold_buttons[index]);

  i=0;  // convert to lower case
  while (str[i] != 0)
  {
    //str[i++] &= ~0x30;
    str[i] = tolower(str[i]);
    ++i;
  }

  rect.top  = 1;
  rect.left = hold_buttons_x[index] + START_X_POS;
  rect.bottom  = rect.top + 16;
  rect.right = rect.left + hold_buttons_size[index];

  if (flags & 1)
  {
    SelectObject(hdc,GetStockObject(WHITE_PEN));
    SelectObject(hdc,GetStockObject(WHITE_BRUSH));
    Rectangle(hdc, START_X_POS, rect.top, 500, rect.bottom);
    return;
  }

  SelectObject(hdc,GetStockObject(BLACK_PEN));
  SelectObject(hdc,GetStockObject(LTGRAY_BRUSH));

  log_brush.lbStyle = 0;  // solid brush
  log_brush.lbColor = RGB(128,128,255);  // blue
  new_brush = CreateBrushIndirect(&log_brush);
  old_brush = (HBRUSH) SelectObject(hdc,new_brush);
  RoundRect(hdc, rect.left, rect.top, rect.right, rect.bottom,6,8);
  SelectObject(hdc,old_brush);
  DeleteObject(new_brush);

  SetBkMode(hdc,TRANSPARENT);
  SetTextColor(hdc, RGB(0,0,0)); // black
  DrawText(hdc, str, strlen(str), &rect, 1);
}

/*------------------------------------------------------------------------
 WCom::update_screen - update anything which needs to be drawn on screen.
   Idea here is that the display data gets dumped into a circular
   buffer which we display to the screen(at a low priority).
|------------------------------------------------------------------------*/
void WCom::update_screen(int all_flag)
{
 int i,len;
 HDC hdc;
 int max_row;
 int write_y;
 int dis_row;
 char *txt;

 HFONT new_hfont = NULL;

#define ROW_HEIGHT 14

  if (scr_size_y < (ROW_HEIGHT))
    return;
    
  max_row = (scr_size_y / ROW_HEIGHT) - 1;
  if (max_row < 1) return;

  hdc = GetDC(hwnd);
      //---- first do fancy light gizmo indicators
  if ((old_msr_status != msr_status) ||
      (old_mcr_status != mcr_status) ||
      all_flag)
  {
    old_msr_status = msr_status;
    old_mcr_status = mcr_status;

    draw_light(hdc, msr_status & ACE_CTS, 0, 0);
    draw_light(hdc, msr_status & ACE_DSR, 1, 0);
    draw_light(hdc, msr_status & ACE_RLSD, 2, 0);
    draw_light(hdc, msr_status & ACE_RI, 3, 0);

    draw_light(hdc, mcr_status & MY_RTS, 4, 0);
    draw_light(hdc, mcr_status & MY_DTR, 5, 0);
    if (num_buttons >= 7)
      draw_light(hdc, mcr_status & MY_LOOPBACK, 6, 0);
  }

  // do holding-reasons indicator lights
  if ( (*((DWORD *)(&old_com_stat)) != *((DWORD *)(&cur_com_stat))) ||
      all_flag)
  {
    draw_hold_light(hdc, 0, 1);  // erase area

    if (cur_com_stat.fCtsHold)
      draw_hold_light(hdc, 0, 0);
    if (cur_com_stat.fDsrHold)
      draw_hold_light(hdc, 1, 0);
    if (cur_com_stat.fRlsdHold)
      draw_hold_light(hdc, 2, 0);
    if (cur_com_stat.fXoffHold)
      draw_hold_light(hdc, 3, 0);
    if (cur_com_stat.fXoffSent)
      draw_hold_light(hdc, 4, 0);
    if (cur_com_stat.fEof)
      draw_hold_light(hdc, 5, 0);
    if (cur_com_stat.fTxim)
      draw_hold_light(hdc, 6, 0);

    //if ( (*((DWORD *)(&old_com_stat)) != *((DWORD *)(&cur_com_stat))) )
    //{
    //  sprintf(gtmpstr, "Hold:%xH[%XH]", *((DWORD *)(&cur_com_stat)),
    //                                    *((DWORD *)(&old_com_stat)) );
    //  mess_str(gtmpstr);
    //}
    old_com_stat = cur_com_stat;
  }

                             // implement almost normal TTY scrolling text
  if (display_cur_row != cur_row)  // force scroll
  {
    display_cur_row = cur_row;
    all_flag = 1;
  }
  dis_row = cur_row;

  for (i=max_row; i>=1; i--)
  {
    if ((text_buf[dis_row][TCOLS+2] != 0) || (all_flag))  // chg flag
    {
      txt = text_buf[dis_row];
      txt[TCOLS+2] = 0;  // reset chg flag
      len = lstrlen(txt);
      write_y = (i*ROW_HEIGHT+4);

      SelectObject(hdc,GetStockObject(WHITE_PEN));
      SelectObject(hdc,GetStockObject(WHITE_BRUSH));
      Rectangle(hdc, 10, write_y, 1600, write_y +ROW_HEIGHT);

      if (len > 0)
        TextOut(hdc, 10, write_y, txt, len);
    }
    if (dis_row > 0) --dis_row;
    else dis_row = TROWS-1;
  }

  ReleaseDC(hwnd, hdc);
}

/*------------------------------------------------------------------------
| WCom::loop_test - perform a loopback test
|------------------------------------------------------------------------*/
void WCom::loop_test(void)
{
 PortStatus ps;
 int stat, failed, got_read;
 DWORD ModemStatus;
 char tmpstr[40];
 char header_str[80];
 DCB org_dcb; // original
 DCB new_dcb;
 int changed_dcb = 0;
 int ri_ok = 0;

  strcpy(header_str, apname);
  strcat(header_str, " - Loopback Test");
  org_dcb.DCBlength = sizeof(DCB);
  new_dcb.DCBlength = sizeof(DCB);

#ifdef WIN32
  if (send_mess_flag)  // sending test string
  {
    send_mess_flag = 0;  // turn off
    CheckMenuItem (hMenuMain,IDM_SENDMESS,  MF_UNCHECKED);
  }

  sprintf(gtmpstr, "Place the loopback plug on %s", com_name);
  if (MessageBox(GetFocus(), gtmpstr, header_str,MB_OKCANCEL) == IDCANCEL)
  {
    return;
  }

  glob_wcom->testing_flag = 1;  // flag to stop main processing while testing

  if (mcr_status & MY_LOOPBACK)
  {
    glob_wcom->LoopBack(0);  // turn off
    mcr_status &= ~MY_LOOPBACK;
  }

#ifdef WIN32
  // kill any data in buffers
  PurgeComm(com_id, PURGE_TXABORT);
  PurgeComm(com_id, PURGE_RXABORT);
#else
  FlushComm(com_id, 0);  // flush tx queue
  FlushComm(com_id, 1);  // flush rx queue
#endif

  Sleep(200);  // wait 200ms

                             // read in the current port settings
  GetCommState(com_id, &org_dcb);

  if ((org_dcb.fDtrControl > 1) || (org_dcb.fRtsControl > 1)) // if auto-handshakes
  {
    GetCommState(com_id, &new_dcb);
    changed_dcb = 1;  // remind to restore
    new_dcb.fRtsControl = 0;
    new_dcb.fDtrControl = 0;
    SetCommState(com_id, &new_dcb);
  }

  failed = 0;

  //------- turn dtr off, rts off
  EscapeCommFunction(com_id, CLRDTR);
  EscapeCommFunction(com_id, CLRRTS);
  mcr_status = (MY_DTR | MY_RTS);
  Sleep(200);  // wait 200ms

  GetCommModemStatus(com_id, &ModemStatus);
  if (ModemStatus & (MS_DSR_ON | MS_RLSD_ON | MS_CTS_ON))
    failed |= 0x0001;

  if ((ModemStatus & MS_RING_ON) == 0)  // do we see ring indicator?
    ri_ok |= 1;

  //------- turn dtr off, rts on
  EscapeCommFunction(com_id, CLRDTR);
  EscapeCommFunction(com_id, SETRTS);
  mcr_status = MY_RTS;
  Sleep(200);  // wait 200ms

  GetCommModemStatus(com_id, &ModemStatus);
  if ((ModemStatus & (MS_DSR_ON | MS_CTS_ON)) != MS_CTS_ON)
    failed |= 0x0002;

  if (ModemStatus & MS_RING_ON)  // do we see ring indicator?
    ri_ok |= 2; // wired to RTS on comtrol loopback.

  //------- turn dtr on, rts off
  EscapeCommFunction(com_id, SETDTR);
  EscapeCommFunction(com_id, CLRRTS);
  mcr_status = MY_DTR;
  Sleep(200);  // wait 200ms

  GetCommModemStatus(com_id, &ModemStatus);
  if ((ModemStatus & (MS_DSR_ON | MS_CTS_ON)) != MS_DSR_ON)
    failed |= 0x0004;

  //------- turn dtr on, rts on
  EscapeCommFunction(com_id, SETDTR);
  EscapeCommFunction(com_id, SETRTS);
  mcr_status = MY_DTR | MY_RTS;
  Sleep(200);  // wait 200ms

  GetCommModemStatus(com_id, &ModemStatus);
  if ((ModemStatus & (MS_DSR_ON | MS_RLSD_ON | MS_CTS_ON)) !=
                     (MS_DSR_ON | MS_RLSD_ON | MS_CTS_ON))
    failed |= 0x0008;

  //---- send some test data
  WritePort((unsigned char *) "TESTING", 8);
  Sleep(200);  // wait 200ms

  // get the com_stat record and get the # of characters available
  StatusPort(&ps);

  if (ps.in_cnt != 8)
  {
    failed |= 0x0010;
  }
  else
  {
    stat = ReadPort((unsigned char *) tmpstr, 8, &got_read);
    if (tmpstr[7] != 0)
      failed |= 0x0020;
    if (tmpstr[0] != 'T')
      failed |= 0x0040;
  }

  if (changed_dcb)
    SetCommState(com_id, &org_dcb);

  if (failed)
  {
    sprintf(gtmpstr, "Port failed loopback test(Error:%xH)", failed);
    MessageBox(GetFocus(), gtmpstr, header_str, MB_OK | MB_ICONSTOP);
  }
  else
  {
    sprintf(gtmpstr, "%s passed the loopback test.", com_name);

    if (ri_ok == 3) // RI is present(give an indication that it was found)
      strcat(gtmpstr, " RI present.");

    MessageBox(GetFocus(), gtmpstr, header_str, MB_OK | MB_ICONINFORMATION);
  }

  glob_wcom->testing_flag = 0;  // flag to stop main processing while testing

#else
   MessageBox(GetFocus(), "loopback not implemented in 16bit version", header_str,MB_OK);
#endif
}

/*------------------------------------------------------------------------
| WCom::clear_scr - clear the screen
|------------------------------------------------------------------------*/
void WCom::clear_scr(void)
{
 int i;

  for (i=0; i<TROWS; i++) // for entire line
  {
    text_buf[i][0] = 0;  // null terminate
    text_buf[cur_row][TCOLS+2] = 1;  // chg flag
  }
  cur_row = 0;
  cur_col = 0;
  update_screen(1);
}

/*------------------------------------------------------------------------
| WCom::mess_str - send a string out to the text window.  Actually just
  dump it into our display circular queue buffer and set a flag to
  tell main poll routine we want to update the screen.  This avoids
  doing scrolling logic here, making it the display tasks problem,
  which is lower priority.
|------------------------------------------------------------------------*/
void WCom::mess_str(char *str, int len)
{
 int i;
 unsigned char c;
 char tmpstr[10];

 if (text_buf == NULL) return;
 if (str == NULL) return;

 if (len == 0)
   len = strlen(str);

 for (i=0; i<len; i++) // for entire line
 {
   c = (unsigned char) str[i];
   switch(c)
   {
     case 0xd:  // CR
       if (show_crlf)
         this->mess_str("<CR>");  // recurse
#ifdef ADD_LF
       if (cur_col != 0)
       {
         ++cur_row;
         if (cur_row >= TROWS)  // round queue
           cur_row = 0;
       }
#endif
       cur_col = 0;
       //  text_buf[cur_row][cur_col] = 0;
       text_buf[cur_row][TCOLS+2] = 1;  // chg flag
     break;

     case 0xa:  // LF
       ++cur_row;
       if (cur_row >= TROWS)  // round queue
         cur_row = 0;
       cur_col = 0;
       text_buf[cur_row][cur_col] = 0;
       text_buf[cur_row][TCOLS+2] = 1;  // chg flag

       if (show_crlf)
         this->mess_str("<LF>");  // recurse
     break;

     default:
       if ((c < 0x20) || (c > 0x80))
       {
         sprintf(tmpstr,"<%xH>",c);
         this->mess_str(tmpstr);  // recurse
         break;
       }
       text_buf[cur_row][cur_col] = c;
       text_buf[cur_row][cur_col+1] = 0;
       text_buf[cur_row][TCOLS+2] = 1;  // chg flag
       ++cur_col;
       if (cur_col >= TCOLS)  // end of line
       {
         cur_col = 0;
         ++cur_row;
         if (cur_row >= TROWS)  // round queue
            cur_row = 0;
         text_buf[cur_row][cur_col] = 0;
         text_buf[cur_row][TCOLS+2] = 1;  // chg flag
       }
     break;
   }
 }
 screen_update_flag = 1;  // need to update screen

}

/*-------------------------------------------------------------------
| SetMiscDlgInfo - Port Options Dialog, setup field values.
|  Misc DCB fields.
|--------------------------------------------------------------------*/
static void SetMiscDlgInfo(HWND hDlg)
{

 unsigned long num;
 int stat;
 DCB *dcb = &glob_dlg_wcom->dcb;

  // common Win16 & Win32
  //------------------ fNull: Enable Null stripping
  dcb->fNull = IsDlgButtonChecked( hDlg, IDC_fNull);

  //------------------ xonchar
  if (GetDlgItemNumb(hDlg, IDC_XonChar, &num) == 0)
    dcb->XonChar = (BYTE) num;

  //------------------ xoffchar
  if (GetDlgItemNumb(hDlg, IDC_XoffChar, &num) == 0)
    dcb->XoffChar = (BYTE) num;

  //------------------ xonlim
  if (GetDlgItemNumb(hDlg, IDC_XonLim, &num) == 0)
    dcb->XonLim = (WORD) num;

  //------------------ xofflim
  if (GetDlgItemNumb(hDlg, IDC_XoffLim, &num) == 0)
    dcb->XoffLim = (WORD) num;

  //------------------ eofchar
  if (GetDlgItemNumb(hDlg, IDC_EofChar, &num) == 0)
    dcb->EofChar = (BYTE) num;

  //------------------ evtchar
  if (GetDlgItemNumb(hDlg, IDC_EvtChar, &num) == 0)
    dcb->EvtChar = (BYTE) num;

#ifdef WIN32
  EnableWindow(GetDlgItem(hDlg, IDC_fBinary),0);
  EnableWindow(GetDlgItem(hDlg, IDC_fPeChar),0);
  EnableWindow(GetDlgItem(hDlg, IDC_fChEvt),0);
  EnableWindow(GetDlgItem(hDlg, IDC_PeChar),0);
  EnableWindow(GetDlgItem(hDlg, IDC_TxDelay),0);

  //------------ Win32 special
  //------------------ DSR Sensitivity
  dcb->fDsrSensitivity = IsDlgButtonChecked( hDlg, IDC_fDsrSensitivity);

  //------------------ Continue TX when Xoff sent
  dcb->fTXContinueOnXoff = IsDlgButtonChecked( hDlg, IDC_fTXContinueOnXoff);

  //------------------ Enable Err Replacement
  dcb->fErrorChar = IsDlgButtonChecked( hDlg, IDC_fErrorChar);

  //------------------ Abort all reads and writes on Error
  dcb->fAbortOnError = IsDlgButtonChecked( hDlg, IDC_fAbortOnError);

  //------------------ Error replacement char
  if (GetDlgItemNumb(hDlg, IDC_ErrorChar, &num) == 0)
    dcb->ErrorChar = (BYTE) num;

  stat = SetCommState(glob_dlg_wcom->com_id,dcb);
  if (stat == TRUE) stat = 0;
  else stat = GetLastError();
#else
  EnableWindow(GetDlgItem(hDlg, IDC_fDsrSensitivity),0);
  EnableWindow(GetDlgItem(hDlg, IDC_fTXContinueOnXoff),0);
  EnableWindow(GetDlgItem(hDlg, IDC_fErrorChar),0);
  EnableWindow(GetDlgItem(hDlg, IDC_fAbortOnError),0);
  EnableWindow(GetDlgItem(hDlg, IDC_ErrorChar),0);

  //------------ Win16 special
  //------------------ fBinary
  dcb->fBinary = IsDlgButtonChecked( hDlg, IDC_fBinary);

  //------------------ fPeChar
  dcb->fPeChar = IsDlgButtonChecked( hDlg, IDC_fPeChar);

  //------------------ fChEvt
  dcb->fChEvt = IsDlgButtonChecked( hDlg, IDC_fChEvt);


  //------------------ PeChar
  if (GetDlgItemNumb(hDlg, IDC_PeChar, &num) == 0)
    dcb->PeChar = (BYTE) num;

   //------------------ TxDelay
  if (GetDlgItemNumb(hDlg, IDC_TxDelay, &num) == 0)
    dcb->TxDelay = (WORD) num;
    stat = SetCommState(dcb);
#endif

  if (stat != 0)
    im_mess1("Error from SetCommState:",stat,0,0);
}

/*-------------------------------------------------------------------
| GetMiscDlgInfo - Port Options Dialog, setup field values.
|  Misc DCB fields.
|--------------------------------------------------------------------*/
static void GetMiscDlgInfo(HWND hDlg)
{
// HWND hwnd;
 char tmpstr[80];
 int stat;
 DCB *dcb = &glob_dlg_wcom->dcb;

                             // read in the current port settings
  stat = GetCommState(glob_dlg_wcom->com_id, dcb);
#ifdef WIN32
  if (stat == TRUE) stat = 0;
  else stat = GetLastError();
#endif
  if (stat != 0)
    im_mess1("Error from GetCommState:",stat,0,0);

  // common Win16 & Win32
  //------------------ fNull: Enable Null stripping
  SendDlgItemMessage(hDlg, IDC_fNull, BM_SETCHECK, dcb->fNull, 0);

  //------------------ xonchar
  sprintf(tmpstr,"%u",dcb->XonChar);
  SetDlgItemText(hDlg, IDC_XonChar, tmpstr);

  //------------------ xoffchar
  sprintf(tmpstr,"%u",dcb->XoffChar);
  SetDlgItemText(hDlg, IDC_XoffChar, tmpstr);

  //------------------ xonlim
  sprintf(tmpstr,"%u",dcb->XonLim);
  SetDlgItemText(hDlg, IDC_XonLim, tmpstr);

  //------------------ xofflim
  sprintf(tmpstr,"%u",dcb->XoffLim);
  SetDlgItemText(hDlg, IDC_XoffLim, tmpstr);

  //------------------ eofchar
  sprintf(tmpstr,"%u",(BYTE) dcb->EofChar);
  SetDlgItemText(hDlg, IDC_EofChar, tmpstr);

  //------------------ evtchar
  sprintf(tmpstr,"%u",(BYTE) dcb->EvtChar);
  SetDlgItemText(hDlg, IDC_EvtChar, tmpstr);

#ifdef WIN32
  //------------ Win32 special
  EnableWindow(GetDlgItem(hDlg, IDC_fBinary),0);
  EnableWindow(GetDlgItem(hDlg, IDC_fPeChar),0);
  EnableWindow(GetDlgItem(hDlg, IDC_fChEvt),0);
  EnableWindow(GetDlgItem(hDlg, IDC_PeChar),0);
  EnableWindow(GetDlgItem(hDlg, IDC_TxDelay),0);
  //------------------ DSR Sensitivity
  SendDlgItemMessage(hDlg, IDC_fDsrSensitivity, BM_SETCHECK, dcb->fDsrSensitivity, 0);

  //------------------ Continue TX when Xoff sent
  SendDlgItemMessage(hDlg, IDC_fTXContinueOnXoff, BM_SETCHECK, dcb->fTXContinueOnXoff, 0);

  //------------------ Enable Err Replacement
  SendDlgItemMessage(hDlg, IDC_fErrorChar, BM_SETCHECK, dcb->fErrorChar, 0);

  //------------------ Abort all reads and writes on Error
  SendDlgItemMessage(hDlg, IDC_fAbortOnError, BM_SETCHECK, dcb->fAbortOnError, 0);

  //------------------ Error replacement char
  sprintf(tmpstr,"%u",(BYTE) dcb->ErrorChar);
  SetDlgItemText(hDlg, IDC_ErrorChar, tmpstr);\

#else
  //------------ Win16 special
  EnableWindow(GetDlgItem(hDlg,IDC_fDsrSensitivity ),0);
  EnableWindow(GetDlgItem(hDlg, IDC_fTXContinueOnXoff),0);
  EnableWindow(GetDlgItem(hDlg, IDC_fErrorChar),0);
  EnableWindow(GetDlgItem(hDlg, IDC_fAbortOnError),0);
  EnableWindow(GetDlgItem(hDlg, IDC_ErrorChar),0);

  //------------------ fBinary
  SendDlgItemMessage(hDlg, IDC_fBinary, BM_SETCHECK, dcb->fBinary, 0);

  //------------------ fPeChar
  SendDlgItemMessage(hDlg, IDC_fPeChar, BM_SETCHECK, dcb->fPeChar, 0);

  //------------------ fChEvt
  SendDlgItemMessage(hDlg, IDC_fChEvt, BM_SETCHECK, dcb->fChEvt, 0);

  //------------------ PeChar
  sprintf(tmpstr,"%u",(BYTE) dcb->PeChar);
  SetDlgItemText(hDlg, IDC_PeChar, tmpstr);

   //------------------ TxDelay
  sprintf(tmpstr,"%u",dcb->TxDelay);
  SetDlgItemText(hDlg, IDC_TxDelay, tmpstr);
#endif
}


/*-----------------------------------------------------------------------------
| init_events_buttons -
|-----------------------------------------------------------------------------*/
void init_events_buttons(HWND hDlg,
#ifdef WIN32
                         DWORD ev,
#else
                         WORD ev,
#endif
                         int rec_cnt, int send_cnt)
{

  SendDlgItemMessage(hDlg, IDC_EV_RXCHAR, BM_SETCHECK, ev & EV_RXCHAR, 0);
  SendDlgItemMessage(hDlg, IDC_EV_RXFLAG, BM_SETCHECK, ev & EV_RXFLAG, 0);
  SendDlgItemMessage(hDlg, IDC_EV_TXEMPTY, BM_SETCHECK, ev & EV_TXEMPTY, 0);
  SendDlgItemMessage(hDlg, IDC_EV_CTS, BM_SETCHECK, ev & EV_CTS, 0);
  SendDlgItemMessage(hDlg, IDC_EV_DSR, BM_SETCHECK, ev & EV_DSR, 0);
  SendDlgItemMessage(hDlg, IDC_EV_RLSD, BM_SETCHECK, ev & EV_RLSD, 0);
  SendDlgItemMessage(hDlg, IDC_EV_BREAK, BM_SETCHECK, ev & EV_BREAK, 0);
  SendDlgItemMessage(hDlg, IDC_EV_ERR, BM_SETCHECK, ev & EV_ERR, 0);
  SendDlgItemMessage(hDlg, IDC_EV_RING, BM_SETCHECK, ev & EV_RING, 0);
  SendDlgItemMessage(hDlg, IDC_EV_PERR, BM_SETCHECK, ev & EV_PERR, 0);
#ifdef WIN32
  EnableWindow(GetDlgItem(hDlg, IDC_EV_CTSS),0);
  EnableWindow(GetDlgItem(hDlg, IDC_EV_DSRS),0);
  EnableWindow(GetDlgItem(hDlg, IDC_EV_RLSDS),0);
  EnableWindow(GetDlgItem(hDlg, IDC_EV_RingTe),0);
  EnableWindow(GetDlgItem(hDlg, IDC_EV_REC_CNT),0);
  EnableWindow(GetDlgItem(hDlg, IDC_EV_SEND_CNT),0);
  SendDlgItemMessage(hDlg, IDC_EV_RX80FULL, BM_SETCHECK, ev & EV_RX80FULL, 0);
  SendDlgItemMessage(hDlg, IDC_EV_EVENT1, BM_SETCHECK, ev & EV_EVENT1, 0);
  SendDlgItemMessage(hDlg, IDC_EV_EVENT2, BM_SETCHECK, ev & EV_EVENT2, 0);
#else
  EnableWindow(GetDlgItem(hDlg, IDC_EV_RX80FULL),0);
  EnableWindow(GetDlgItem(hDlg, IDC_EV_EVENT1),0);
  EnableWindow(GetDlgItem(hDlg, IDC_EV_EVENT2),0);
  SendDlgItemMessage(hDlg, IDC_EV_CTSS, BM_SETCHECK, ev & EV_CTSS, 0);
  SendDlgItemMessage(hDlg, IDC_EV_DSRS, BM_SETCHECK, ev & EV_DSRS, 0);
  SendDlgItemMessage(hDlg, IDC_EV_RLSDS, BM_SETCHECK, ev & EV_RLSDS, 0);
  SendDlgItemMessage(hDlg, IDC_EV_RingTe, BM_SETCHECK, ev & EV_RingTe, 0);
  {
    char tmpstr[20];
    sprintf(tmpstr,"%d",rec_cnt);
    SetDlgItemText(hDlg, IDC_EV_REC_CNT, tmpstr);

    sprintf(tmpstr,"%d",send_cnt);
    SetDlgItemText(hDlg, IDC_EV_SEND_CNT, tmpstr);
  }

#endif
}


/*-----------------------------------------------------------------------------
| GetDlgItemNumb - get a number from the text dialog field
|-----------------------------------------------------------------------------*/
static int GetDlgItemNumb(HWND hDlg, int ctrl, unsigned long *num)
{
 char tmpstr[40];
 int i;


  GetDlgItemText(hDlg, ctrl, tmpstr, 18);
  i = 0;
  while (tmpstr[i] == ' ')
    ++i;
         // look for leading 0x for hex numbers
  if ((tmpstr[i] == '0') && (toupper(tmpstr[i+1]) == 'X'))
  {
    i+=2;
    if (sscanf(&tmpstr[i],"%lx",num) == 1)
      return 0;
    return 1;
  }

         // look for trailing 'H' for hex numbers
  i = strlen(tmpstr);
  if (i>0) --i;
  while ((tmpstr[i] == ' ') && (i != 0))
    --i;
  if ((toupper(tmpstr[i])) == 'H')
  {
    if (sscanf(tmpstr,"%lx",num) == 1)
      return 0;
    return 1;
  }

        // normal decimal case
  if (sscanf(tmpstr,"%ld",num) == 1)
    return 0;  // ok

  if (tmpstr[0] != 0)  // allow ascii to dec conversion case
    *num = tmpstr[0];
  return 0;  // ok
}


/*-------------------------------------------------------------------
| SetSettingsDlgInfo - Port Options Dialog, setup field values.
|--------------------------------------------------------------------*/
static void SetSettingsDlgInfo(HWND hDlg)
{

 int j, stat;
 DCB *dcb = &glob_dlg_wcom->dcb;
 char tmpstr[60];
 DWORD val;

  //------------------ baud
  GetDlgItemText(hDlg, IDCOMBAUD, tmpstr, 58);
  stat= sscanf(tmpstr,"%ld",&val);
  if ((stat == 1) && (val >= 0))
    dcb->BaudRate = (UINT) val;

  //------------------ bytes size
  GetDlgItemText(hDlg, IDCOMDATA, tmpstr, 58);
  stat= sscanf(tmpstr,"%ld",&val);
  if ((stat == 1) && (val >= 0))
  {
		char Rhee[200];
		dcb->ByteSize = (BYTE) val;
		//wsprintf(Rhee, "ByteSize = %d", dcb->ByteSize);
		//MessageBox(hDlg, Rhee, 0,0);
  }

  //------------------ stop-bits
  j = (int)SendDlgItemMessage(hDlg, IDCOMSTOP, CB_GETCURSEL, 0, 0L);
  dcb->StopBits = (BYTE) j;

  //------------------ parity
  j = (int)SendDlgItemMessage(hDlg, IDCOMPARITY, CB_GETCURSEL, 0, 0L);
  dcb->Parity = j;
/*  if (dcb->Parity != 0)
    dcb->fParity = 1;*/

  // RTS Ctrl
  j = (int)SendDlgItemMessage(hDlg, IDCOMRTS, CB_GETCURSEL, 0,0L);
  dcb->fRtsControl = j;
  switch(j)	// Keep Track of output pins as best we can
  {
	case 0:		// Disable				
		glob_dlg_wcom->mcr_status &= ~MY_RTS;
		break;
	case 1:		// Enable
		glob_dlg_wcom->mcr_status |= MY_RTS;
		break;
	case 2:		// HandShake
		glob_dlg_wcom->mcr_status |= MY_RTS;
		break;
	case 3:		// Toggle
		glob_dlg_wcom->mcr_status |= MY_RTS;
		break;
  }

  // DTR Ctrl
  j = (int)SendDlgItemMessage(hDlg, IDCOMDTR, CB_GETCURSEL, 0,0L);
  dcb->fDtrControl=j;
  switch(j)	// Keep Track of output pins as best we can
  {
	case 0:		// Clear
		glob_dlg_wcom->mcr_status &= ~MY_DTR;
		break;
	case 1:		// Set
		glob_dlg_wcom->mcr_status |= MY_DTR;
		break;
	case 2:		// HandShake (Auto)
		glob_dlg_wcom->mcr_status |= MY_DTR;
		break;
  }

  // Xon-Xoff out,in
  dcb->fOutX = IsDlgButtonChecked (hDlg, IDCOMSENDXON);
  dcb->fInX  = IsDlgButtonChecked (hDlg, IDCOMRECXON);

  // cts,dsr
  dcb->fOutxCtsFlow = IsDlgButtonChecked(hDlg, IDCOMCTS);
  dcb->fOutxDsrFlow = IsDlgButtonChecked(hDlg, IDCOMDSR);

  stat = SetCommState(glob_dlg_wcom->com_id,dcb);
  if (stat == TRUE)
		stat=0;
  else
		stat=GetLastError();

  if (stat != 0)
		im_mess1("Error from SetCommState:",stat,0,0);

}

/*-------------------------------------------------------------------
| GetSettingsDlgInfo - Port Options Dialog, setup field values.
|--------------------------------------------------------------------*/
static void GetSettingsDlgInfo(HWND hDlg)
{
 HWND hwnd;
 char tmpstr[80];
 int j, found;
 int stat;
 DCB *dcb = &glob_dlg_wcom->dcb;
 static DWORD bauds[] = {300,600,1200,2400,3600,4800,7200,9600,19200,
                         38400, 57600, 115200, 230400, 460800, 921600, 0};

                             // read in the current port settings
  stat = GetCommState(glob_dlg_wcom->com_id, dcb);
#ifdef WIN32
  if (stat == TRUE) stat = 0;
  else stat = GetLastError();
#endif
  if (stat != 0)
    im_mess1("Error from GetCommState:",stat,0,0);

  //------------------ fill in baud rate cb
  hwnd = GetDlgItem(hDlg, IDCOMBAUD);
  j=0;
  found = 0;
  SendMessage(hwnd, CB_RESETCONTENT, 0, 0);
  while (bauds[j] != 0)
  {
    if (dcb->BaudRate == bauds[j])
      found = j;
    sprintf(tmpstr,"%u",bauds[j++]);
    SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) tmpstr);
  }
  SendMessage(hwnd, CB_SETCURSEL, found, (LPARAM)0);

  //------------------ fill in data bits cb
  hwnd = GetDlgItem(hDlg, IDCOMDATA);
  SendMessage(hwnd, CB_RESETCONTENT, 0, 0);
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "7 bits");
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "8 bits");
  if (dcb->ByteSize == 7)
       SendMessage(hwnd, CB_SETCURSEL, 0, (LPARAM)0);
  else SendMessage(hwnd, CB_SETCURSEL, 1, (LPARAM)0);

 //------------------ fill in parity cb
  hwnd = GetDlgItem(hDlg, IDCOMPARITY);
  SendMessage(hwnd, CB_RESETCONTENT, 0, 0);
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "None");
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Odd");
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Even");
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Mark");
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Space");
  SendMessage(hwnd, CB_SETCURSEL, dcb->Parity, (LPARAM)0);

  //------------------ fill in stop bits
  hwnd = GetDlgItem(hDlg, IDCOMSTOP);
  SendMessage(hwnd, CB_RESETCONTENT, 0, 0);
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "1");
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "1.5");
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "2");
  SendMessage(hwnd, CB_SETCURSEL, dcb->StopBits, (LPARAM)0);

  //------------------ fill in flow
  hwnd = GetDlgItem(hDlg, IDCOMRTS);
  SendMessage(hwnd, CB_RESETCONTENT, 0, 0);
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Clear");
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Set");
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Auto");
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Toggle");
  SendMessage(hwnd, CB_SETCURSEL, dcb->fRtsControl, (LPARAM)0);

  hwnd = GetDlgItem(hDlg, IDCOMDTR);
  SendMessage(hwnd, CB_RESETCONTENT, 0, 0);
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Clear");
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Set");
  SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Auto");
  SendMessage(hwnd, CB_SETCURSEL, dcb->fDtrControl, (LPARAM)0);

  if (dcb->fInX)
  {
		hwnd = GetDlgItem(hDlg, IDCOMRECXON);
  		SendMessage(hwnd, BM_SETCHECK, BST_CHECKED, 0);
  }
  
  if (dcb->fOutxCtsFlow)
  {
		hwnd = GetDlgItem(hDlg, IDCOMCTS);
		SendMessage(hwnd, BM_SETCHECK, BST_CHECKED, 0);
  }

  if (dcb->fOutxDsrFlow)
  {
		hwnd = GetDlgItem(hDlg, IDCOMDSR);
  		SendMessage(hwnd, BM_SETCHECK, BST_CHECKED, 0);
  }

  if (dcb->fOutX)
  {
		hwnd = GetDlgItem(hDlg, IDCOMSENDXON);
  		SendMessage(hwnd, BM_SETCHECK, BST_CHECKED, 0);
  }

  return;
}

/*-----------------------------------------------------------------------------
| settings_dlg_proc - Port Options dialog screen.
|-----------------------------------------------------------------------------*/
BOOL FAR PASCAL settings_dlg_proc(HWND hDlg, UINT message,
                         WPARAM wParam, LPARAM lParam)
{

  switch (message)
  {
    case WM_INITDIALOG:     /* message: initialize dialog box */
      GetSettingsDlgInfo(hDlg);
    return 0;

    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case IDOK:

			if (gfSelectAllPorts) {
			  WCom *wcomorg = glob_dlg_wcom; 
			  glob_dlg_wcom = head_wcom;

			  while (glob_dlg_wcom != NULL)
			  {
				SetSettingsDlgInfo(hDlg);
				glob_dlg_wcom = glob_dlg_wcom->next_wcom;
			  }
			  glob_dlg_wcom = wcomorg;
			  gfSelectAllPorts = 0;
			}
			else 
	          SetSettingsDlgInfo(hDlg);
          
			EndDialog(hDlg, NULL);       /* Exits the dialog box      */
        break;

        case IDCANCEL:
          EndDialog(hDlg, NULL);       /* Exits the dialog box      */
        break;

		case IDC_BUTTON_RESET:
			{
			DWORD nBytes;
			ULONG LineStatus = 0;
			char tmpString[8];

			/*DeviceIoControl(glob_wcom->com_id, IOCTL_RESET_MODEM,
                         (char *)NULL, 0, (char *)NULL, 0,  // out
                         &nBytes, 0);
			*/
			DeviceIoControl(glob_wcom->com_id, IOCTL_OXSER_READ_LSR,
                         (char *)NULL, 0, &LineStatus, sizeof(ULONG),  // out
                         &nBytes, 0);
			sprintf(tmpString, "0x%x", LineStatus);
			MessageBox(0, tmpString, 0, 0);
			}
			break;
	  
	  }
    break;
  }
  return (FALSE);         /* Didn't process a message    */
}

/*-----------------------------------------------------------------------------
| OpenPortProc - Opening dialog screen.
|-----------------------------------------------------------------------------*/
BOOL FAR PASCAL OpenPortProc(HWND hDlg, UINT message,
                         WPARAM wParam, LPARAM lParam)
{

	MDICREATESTRUCT    mdicreate;
	char tmpstr[255];
	unsigned StartPort,Amount,PortNum;
	int stat,j;
	DWORD val;

  switch (message)
  {
    case WM_INITDIALOG:     /* message: initialize dialog box */
		{
			int j=0;
			int found=0;
			HWND hwnd;
			char tmpstr[80];

			static DWORD bauds[] = {300,600,1200,2400,3600,4800,7200,9600,19200,
									38400,57600,115200,230400,460800,921600,0};
			hwnd = GetDlgItem(hDlg, IDCOMBAUD);
			SendMessage(hwnd, CB_RESETCONTENT, 0,0);
			while(bauds[j] != 0)
			{
			    sprintf(tmpstr,"%u",bauds[j++]);
				SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) tmpstr);
			}
			SendMessage(hwnd, CB_SETCURSEL, 7, (LPARAM)0);

			for (j=1; j<256; j++)
			{
				wsprintf(tmpstr, "COM%d", j);
				SendDlgItemMessage(hDlg, IDCOMFROM, CB_ADDSTRING,0, (LPARAM)tmpstr);
			}
			SendDlgItemMessage(hDlg, IDCOMFROM, CB_SETCURSEL,4, (LPARAM)0);

			hwnd = GetDlgItem(hDlg, IDCOMQUANTITY);
			SendMessage(hwnd, CB_RESETCONTENT, 0, 0);
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "1 Ports");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "2 Ports");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "4 Ports");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "8 Ports");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "16 Ports");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "24 Ports");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "32 Ports");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "64 Ports");
			SendMessage(hwnd, CB_SETCURSEL, 0, (LPARAM)0);
			
				//------------------ fill in data bits cb
			hwnd = GetDlgItem(hDlg, IDCOMDATA);
			SendMessage(hwnd, CB_RESETCONTENT, 0, 0);
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "7 bits");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "8 bits");
			SendMessage(hwnd, CB_SETCURSEL, 1, (LPARAM)0);

				//------------------ fill in parity cb
			hwnd = GetDlgItem(hDlg, IDCOMPARITY);
			SendMessage(hwnd, CB_RESETCONTENT, 0, 0);
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "None");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Odd");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Even");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Mark");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Space");
			SendMessage(hwnd, CB_SETCURSEL, 0, (LPARAM)0);

				//------------------ fill in stop bits
			hwnd = GetDlgItem(hDlg, IDCOMSTOP);
			SendMessage(hwnd, CB_RESETCONTENT, 0, 0);
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "1");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "1.5");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "2");
			SendMessage(hwnd, CB_SETCURSEL, 0, (LPARAM)0);

				//------------------ fill in RTS-flow
			hwnd = GetDlgItem(hDlg, IDCOMRTS);
			SendMessage(hwnd, CB_RESETCONTENT, 0, 0);
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Clear");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Set");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Auto");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Toggle");
			SendMessage(hwnd, CB_SETCURSEL, 1, (LPARAM)0);

				//------------------ fill in DTR-Flow
			hwnd = GetDlgItem(hDlg, IDCOMDTR);
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Clear");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Set");
			SendMessage(hwnd, CB_ADDSTRING, 0, (LPARAM) "Auto");
			SendMessage(hwnd, CB_SETCURSEL, 1, (LPARAM)1);
		}
    return 0;

    case WM_COMMAND:
      switch (LOWORD(wParam))
      {
        case IDOK:
			EndDialog(hDlg, NULL);       /* Exits the dialog box      */

			StartPort = Amount = 0;
			GetDlgItemText(hDlg, IDCOMFROM, tmpstr, 58);
			StartPort = atoi(&tmpstr[3]);
			GetDlgItemText(hDlg, IDCOMQUANTITY, tmpstr, 58);
			Amount = atoi(tmpstr);

			for (PortNum = StartPort; PortNum < StartPort + Amount ; PortNum++)
			{
				
		        mdicreate.x       = CW_USEDEFAULT;
		        mdicreate.y       = CW_USEDEFAULT;
		        mdicreate.cx      = CW_USEDEFAULT;
		        mdicreate.cy      = CW_USEDEFAULT;
		        mdicreate.szClass = "WcomDraw";
		        mdicreate.szTitle = "ComX";
		        mdicreate.hOwner  = glob_hinst ;
		//      mdicreate.style   =  WS_VSCROLL | WS_HSCROLL;
		        mdicreate.style = 0;
		        mdicreate.lParam  = 0;

		        glob_wcom = new WCom;
				sprintf(gtmpstr,"COM%d", PortNum);
		//		MessageBox(hDlg,gtmpstr,0,0);

		        stat = glob_wcom->OpenPort(gtmpstr);
		        if (stat != 0)
				{
					MessageBox(GetFocus(),"Port failed to open.",0,MB_OK);
					continue;
				}

				// enable settings menu
		        EnableMenuItem(hMenuMain,1,MF_BYPOSITION | MF_ENABLED);
				// enable windows menu
				EnableMenuItem(hMenuMain,2,MF_BYPOSITION | MF_ENABLED);

				// create the mdi window
				hwndDraw = (HWND) SendMessage (hwndClient, WM_MDICREATE, (WPARAM) 0,
                              (LPARAM) (LONG) (LPMDICREATESTRUCT) &mdicreate);

				if (hwndDraw == NULL)
				{
					MessageBox(GetFocus(),"Port No Open.","",MB_OK);
					return 0;
				}

				glob_dlg_wcom = glob_wcom;
				DCB *dcb = &glob_dlg_wcom->dcb;

				// Baud
				GetDlgItemText(hDlg, IDCOMBAUD, tmpstr, 58);
				stat = sscanf(tmpstr, "%ld",&val);
				if ((stat == 1) && (val >= 0))
					dcb->BaudRate = (UINT) val;

				// Data Bits
				GetDlgItemText(hDlg, IDCOMDATA, tmpstr, 58);
				stat = sscanf(tmpstr, "%ld",&val);
				if ((stat == 1) && (val >= 0))
					dcb->ByteSize = (BYTE) val;

				// Stop Bits
				j = (int)SendDlgItemMessage(hDlg, IDCOMSTOP, CB_GETCURSEL, 0, 0L);
				dcb->StopBits = (BYTE) j;

				// Parity
				j = (int)SendDlgItemMessage(hDlg, IDCOMPARITY, CB_GETCURSEL, 0, 0L);
				dcb->Parity = j;
/*				if (dcb->Parity != 0)
					dcb->Parity = 1;*/

				// RTS Ctrl
				j = (int)SendDlgItemMessage(hDlg, IDCOMRTS, CB_GETCURSEL, 0,0L);
				dcb->fRtsControl = j;
				switch(j)	// Keep Track of output pins as best we can
				{
				case 0:
					glob_dlg_wcom->mcr_status &= ~MY_RTS;
					break;
				case 1:
					glob_dlg_wcom->mcr_status |= MY_RTS;
					break;
				case 2:
					glob_dlg_wcom->mcr_status |= MY_RTS;
					break;
				case 3:
					glob_dlg_wcom->mcr_status |= MY_RTS;
					break;
				}

				// DTR Ctrl
				j = (int)SendDlgItemMessage(hDlg, IDCOMDTR, CB_GETCURSEL, 0,0L);
				dcb->fDtrControl=j;
				switch(j)	// Keep Track of output pins as best we can
				{
				case 0:
					glob_dlg_wcom->mcr_status &= ~MY_DTR;
					break;
				case 1:
					glob_dlg_wcom->mcr_status |= MY_DTR;
					break;
				case 2:
					glob_dlg_wcom->mcr_status |= MY_DTR;
					break;
				}

				// Xon-Xoff out,in
				dcb->fOutX = IsDlgButtonChecked (hDlg, IDCOMSENDXON);
				dcb->fInX  = IsDlgButtonChecked (hDlg, IDCOMRECXON);

				// cts,dsr
				dcb->fOutxCtsFlow = IsDlgButtonChecked(hDlg, IDCOMCTS);
				dcb->fOutxDsrFlow = IsDlgButtonChecked(hDlg, IDCOMDSR);

				stat = SetCommState(glob_dlg_wcom->com_id,dcb);
				if (stat == TRUE)
					stat=0;
				else
					stat=GetLastError();

				if (stat != 0)
					im_mess1("Error from SetCommState:",stat,0,0);

			}
			//          SetSettingsDlgInfo(hDlg);
		    break;

        case IDCANCEL:
			EndDialog(hDlg, NULL);       /* Exits the dialog box      */
			break;

      }
	  break;
  }
  return (FALSE);         /* Didn't process a message    */
}

#ifdef COMMENT_OUT
*******************************************************************

************** 32 bit, if __FLAT__  (in winbase.h)
typedef struct _DCB {
         DWORD DCBlength;      /* sizeof(DCB)                     */
         DWORD BaudRate;       /* Baudrate at which running       */
         DWORD fBinary: 1;     /* Binary Mode (skip EOF check)    */
         DWORD fParity: 1;     /* Enable parity checking          */
         DWORD fOutxCtsFlow:1; /* CTS handshaking on output       */
         DWORD fOutxDsrFlow:1; /* DSR handshaking on output       */
         DWORD fDtrControl:2;  /* DTR Flow control                */
    DWORD fDsrSensitivity:1; /* DSR Sensitivity              */
    DWORD fTXContinueOnXoff: 1; /* Continue TX when Xoff sent */
    DWORD fOutX: 1;       /* Enable output X-ON/X-OFF        */
    DWORD fInX: 1;        /* Enable input X-ON/X-OFF         */
    DWORD fErrorChar: 1;  /* Enable Err Replacement          */
         DWORD fNull: 1;       /* Enable Null stripping           */
    DWORD fRtsControl:2;  /* Rts Flow control                */
    DWORD fAbortOnError:1; /* Abort all reads and writes on Error */
    DWORD fDummy2:17;     /* Reserved                        */
    WORD wReserved;       /* Not currently used              */
    WORD XonLim;          /* Transmit X-ON threshold         */
    WORD XoffLim;         /* Transmit X-OFF threshold        */
    BYTE ByteSize;        /* Number of bits/byte, 4-8        */
    BYTE Parity;          /* 0-4=None,Odd,Even,Mark,Space    */
    BYTE StopBits;        /* 0,1,2 = 1, 1.5, 2               */
    char XonChar;         /* Tx and Rx X-ON character        */
    char XoffChar;        /* Tx and Rx X-OFF character       */
    char ErrorChar;       /* Error replacement char          */
         char EofChar;         /* End of Input character          */
         char EvtChar;         /* Received Event character        */
         WORD wReserved1;      /* Fill for now.                   */
} DCB, *LPDCB;

//
// DTR Control Flow Values.
//
#define DTR_CONTROL_DISABLE    0x00
#define DTR_CONTROL_ENABLE     0x01
#define DTR_CONTROL_HANDSHAKE  0x02

//
// RTS Control Flow Values
//
#define RTS_CONTROL_DISABLE    0x00
#define RTS_CONTROL_ENABLE     0x01
#define RTS_CONTROL_HANDSHAKE  0x02
#define RTS_CONTROL_TOGGLE     0x03

#endif

